# Misc. Utility Functions [(API)](https://public.msli.com/lcs/muscle/html/group__misutilityfunctions.html)

```#include "util/MiscUtilityFunctions.h"```

Various functions that didn't fit well anywhere else are kept here.

* [ParseArgs()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga252c53847c1f7472f3952353823048d6) and [ParseFile()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga53e933a2aac7a2182c249165d98094d3) parse a textual list of arguments (of the form `key` or `key=value`) into a [Message](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) object for easy argument-parsing.  The input can be in the form of a single string or a file.
* [UnparseArgs()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga2b01dba074b1ebcb5239aaaf61a62396) and [UnparseFile()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga3a1a0c0c00f0f0d54c7b9ec32d4c7c54) do the opposite -- converts a [Message](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) object's string fields back into a text-string that can be written to disk as a config file.
* [ParseConnectArg()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#gab6d5b6b1a2c4c462d196ad66c0e04494) takes a [String](https://public.msli.com/lcs/muscle/html/classmuscle_1_1String.html) of the form "hostname:port" and parses out the hostname and port into separate fields.
* [HandleStandardDaemonArgs()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga7d0f2c20a86c350ca915cd1a4f02f034) takes a [Message](https://public.msli.com/lcs/muscle/html/classmuscle_1_1Message.html) of command line / config arguments and looks for common arguments to handle in the standard way (e.g. "displaylevel=debug", "console", "nice", etc)
* [Crash()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga84b6f4b71fe1874538f70462d116e368) does what the name suggests it will do.
* [BecomeDaemonProcess()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga97bdf2d620bcc6ec76861ce5f440506e) transfers the current process into the background, detached it from its parent process/shell.
* [SpawnDaemonProcess()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga687d65fbd80774a144707e68b11edc0b) spawns a child process that will run in the background as a fork()'d copy of the current process.
* [NybbleizeData()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#gac28180c792270fea25ce286fcaa62fb7) converts binary data into an ASCII-compatible 4-bits-per-character "safe" string, for passing the data over non-binary-safe transports.  [DenybbleizeString()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga8146ada678bc97e2d266d53c86a8f610) does the opposite.
* [MemMem()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga66f897123d0b965eeaa80351b6a7193e) looks for the first occurrence small binary sequence within a larger one.  (Like `strstr()`, except for raw binary)
* [PrintHexBytes()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#gadd75bc158b9e439e5cdaad6c127e3fe3) prints out the specified binary data, in a pretty hexadecimal format, annotated with an ASCII sidebar (useful for debugging)
* [LogHexBytes()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga90de31c013786f4174a3251e0c8f5560) does the same thing as [PrintHexBytes()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#gadd75bc158b9e439e5cdaad6c127e3fe3), except the output goes to the log instead of just to stdout.
* [HexBytesToAnnotatedString()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga62a39ff27e6b85be88d6f7133a752b34) does the same thing again, except the output is returned as a [String](https://public.msli.com/lcs/muscle/html/classmuscle_1_1String.html) instead of output anywhere.
* [ParseHexBytes()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#gad934b1623728bd533f25783b955c4783) reads a human-readable string containing hex printouts (e.g. "1f 7e 26 35") and returns the corresponding binary data as a [ByteBufferRef](https://public.msli.com/lcs/muscle/html/classmuscle_1_1ByteBuffer.html).
* [HexBytesToString()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga91a3a522cf5ce02357e4192f668441bd) converts a raw binary sequence into a human-readable string (e.g. "1f 7e 26 35"), without any annotation.
* [FileExists()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#gae5b2f3ebdfe65b3126e6ea79a8c0e82c), [RenameFile()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#gaf6efa8691d3c50e7a0a104422b6cb658), [CopyFile()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#gae9f7a79f96b90d50f5623ba2217e8eba), and [DeleteFile()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga1265d28c1df5c814a265c08bb0182551) all do what their names suggest.
* `Win32AllocateStdioConsole()` opens a DOS prompt so that stdout output can be viewed (under Windows only; other platforms don't need this)
* [GetSystemMemoryUsagePercentage()](https://public.msli.com/lcs/muscle/html/group__miscutilityfunctions.html#ga0282a38fcf3a54bc9897bad966783438) returns the percentage of the host computer's RAM that is currently in use.

Quick links to source code of relevant MUSCLE-by-example programs:

* [miscutilityfunctions/example_1_monotonic_clock.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/miscutilityfunctions/example_1_command_line_args.cpp)
* [miscutilityfunctions/example_2_calendar_misc.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/miscutilityfunctions/example_2_config_file.cpp)
* [miscutilityfunctions/example_3_hex_bytes.cpp](https://public.msli.com/lcs/muscle/muscle/html/muscle-by-example/examples/miscutilityfunctions/example_3_hex_bytes.cpp)
